@@ -14,7 +14,7 @@
 #if V8_OS_LINUX
 #include <linux/auxvec.h>  // AT_HWCAP
 #endif
-#if V8_GLIBC_PREREQ(2, 16) || V8_OS_ANDROID
+#if V8_OS_LINUX
 #include <sys/auxv.h>  // getauxval()
 #endif
 #if V8_OS_QNX
@@ -190,17 +190,20 @@ static V8_INLINE void __cpuidex(int cpu_info[4], int info_type,
  * HWCAP2 flags - for elf_hwcap2 (in kernel) and AT_HWCAP2
  */
 #define HWCAP2_MTE (1 << 18)
+#define HWCAP2_CSSC (1UL << 34)
+#define HWCAP2_MOPS (1UL << 43)
+#define HWCAP2_HBC (1UL << 44)
 #endif  // V8_HOST_ARCH_ARM64
 
 #if V8_HOST_ARCH_ARM || V8_HOST_ARCH_ARM64
 
-static std::tuple<uint32_t, uint32_t> ReadELFHWCaps() {
-  uint32_t hwcap = 0;
-  uint32_t hwcap2 = 0;
-#if (V8_GLIBC_PREREQ(2, 16) || V8_OS_ANDROID) && defined(AT_HWCAP)
-  hwcap = static_cast<uint32_t>(getauxval(AT_HWCAP));
+static std::tuple<uint64_t, uint64_t> ReadELFHWCaps() {
+  uint64_t hwcap = 0;
+  uint64_t hwcap2 = 0;
+#if V8_OS_LINUX && defined(AT_HWCAP)
+  hwcap = static_cast<uint64_t>(getauxval(AT_HWCAP));
 #if defined(AT_HWCAP2)
-  hwcap2 = static_cast<uint32_t>(getauxval(AT_HWCAP2));
+  hwcap2 = static_cast<uint64_t>(getauxval(AT_HWCAP2));
 #endif  // AT_HWCAP2
 #else
   // Read the ELF HWCAP flags by parsing /proc/self/auxv.
@@ -445,8 +448,12 @@ CPU::CPU()
       has_dot_prod_(false),
       has_lse_(false),
       has_mte_(false),
+      has_sha3_(false),
       has_pmull1q_(false),
       has_fp16_(false),
+      has_hbc_(false),
+      has_cssc_(false),
+      has_mops_(false),
       is_fp64_mode_(false),
       has_non_stop_time_stamp_counter_(false),
       is_running_in_vm_(false),
@@ -708,7 +715,8 @@ CPU::CPU()
   }
 
   // Try to extract the list of CPU features from ELF hwcaps.
-  uint32_t hwcaps, hwcaps2;
+  uint64_t hwcaps = 0;
+  uint64_t hwcaps2 = 0;
   std::tie(hwcaps, hwcaps2) = ReadELFHWCaps();
   if (hwcaps != 0) {
     has_idiva_ = (hwcaps & HWCAP_IDIVA) != 0;
@@ -831,15 +839,19 @@ CPU::CPU()
 
 #elif V8_OS_LINUX
   // Try to extract the list of CPU features from ELF hwcaps.
-  uint32_t hwcaps, hwcaps2;
+  uint64_t hwcaps, hwcaps2;
   std::tie(hwcaps, hwcaps2) = ReadELFHWCaps();
+  has_cssc_ = (hwcaps2 & HWCAP2_CSSC) != 0;
   has_mte_ = (hwcaps2 & HWCAP2_MTE) != 0;
+  has_hbc_ = (hwcaps2 & HWCAP2_HBC) != 0;
+  has_mops_ = (hwcaps2 & HWCAP2_MOPS) != 0;
   if (hwcaps != 0) {
     has_jscvt_ = (hwcaps & HWCAP_JSCVT) != 0;
     has_dot_prod_ = (hwcaps & HWCAP_ASIMDDP) != 0;
     has_lse_ = (hwcaps & HWCAP_ATOMICS) != 0;
     has_pmull1q_ = (hwcaps & HWCAP_PMULL) != 0;
     has_fp16_ = (hwcaps & HWCAP_FPHP) != 0;
+    has_sha3_ = (hwcaps & HWCAP_SHA3) != 0;
   } else {
     // Try to fallback to "Features" CPUInfo field
     CPUInfo cpu_info;
@@ -849,6 +861,7 @@ CPU::CPU()
     has_lse_ = HasListItem(features, "atomics");
     has_pmull1q_ = HasListItem(features, "pmull");
     has_fp16_ = HasListItem(features, "half");
+    has_sha3_ = HasListItem(features, "sha3");
     delete[] features;
   }
 #elif V8_OS_DARWIN
@@ -893,6 +906,14 @@ CPU::CPU()
   } else {
     has_fp16_ = fp16;
   }
+  int64_t feat_sha3 = 0;
+  size_t feat_sha3_size = sizeof(feat_sha3);
+  if (sysctlbyname("hw.optional.arm.FEAT_SHA3", &feat_sha3, &feat_sha3_size,
+                   nullptr, 0) == -1) {
+    has_sha3_ = false;
+  } else {
+    has_sha3_ = feat_sha3;
+  }
 #else
   // ARM64 Macs always have JSCVT, ASIMDDP, FP16 and LSE.
   has_jscvt_ = true;
@@ -900,6 +921,7 @@ CPU::CPU()
   has_lse_ = true;
   has_pmull1q_ = true;
   has_fp16_ = true;
+  has_sha3_ = true;
 #endif  // V8_OS_IOS
 #endif  // V8_OS_WIN
 
@@ -934,7 +956,9 @@ CPU::CPU()
 
   part_ = -1;
   if (auxv_cpu_type) {
-    if (strcmp(auxv_cpu_type, "power10") == 0) {
+    if (strcmp(auxv_cpu_type, "power11") == 0) {
+      part_ = kPPCPower11;
+    } else if (strcmp(auxv_cpu_type, "power10") == 0) {
       part_ = kPPCPower10;
     } else if (strcmp(auxv_cpu_type, "power9") == 0) {
       part_ = kPPCPower9;
